@@ -260,8 +260,11 @@ if __name__ == '__main__':
 
     # Must be done before connecting to D-Bus (for some reason).
     if not pynotify.init (PROGRAM_NAME):
-        print >> sys.stderr, ("%s: unable to initialize pynotify" %
-                              PROGRAM_NAME)
+        try:
+            print >> sys.stderr, ("%s: unable to initialize pynotify" %
+                                  PROGRAM_NAME)
+        except:
+            pass
 
     if trayicon:
         # Stop running when the session ends.
@@ -288,16 +291,25 @@ if __name__ == '__main__':
             NewPrinterNotification(bus)
             service_running = True
         except:
-            print >> sys.stderr, \
-                "%s: failed to start NewPrinterNotification service" % \
-                PROGRAM_NAME
+            try:
+                print >> sys.stderr, \
+                    "%s: failed to start NewPrinterNotification service" % \
+                    PROGRAM_NAME
+            except:
+                pass
 
     if trayicon and get_debugging () == False:
         # Start off just waiting for print jobs.
         def any_jobs ():
             try:
                 c = cups.Connection ()
-                if len (c.getJobs (my_jobs=True)):
+                try:
+                    jobs = c.getJobs (my_jobs=True, limit=1)
+                except TypeError:
+                    # limit parameter requires pycups >= 1.9.42.
+                    jobs = c.getJobs (my_jobs=True)
+
+                if len (jobs):
                     return True
             except:
                 pass
