@@ -93,10 +93,22 @@ class AuthDialog(gtk.Dialog):
         self.field_entry[i].grab_focus ()
 
 class Connection:
-    def __init__ (self, parent=None, try_as_root=True):
+    def __init__ (self, parent=None, try_as_root=True,
+                  host=None, port=None, encryption=None):
+        if host != None:
+            cups.setServer (host)
+        if port != None:
+            cups.setPort (port)
+        if encryption != None:
+            cups.setEncryption (encryption)
+
         self._use_password = ''
         self._parent = parent
         self._try_as_root = try_as_root
+        self._use_user = cups.getUser ()
+        self._server = cups.getServer ()
+        self._port = cups.getPort()
+        self._encryption = cups.getEncryption ()
         self._connect ()
         self._prompt_allowed = True
 
@@ -107,9 +119,18 @@ class Connection:
         self._prompt_allowed = allowed
 
     def _connect (self):
-        self._connection = cups.Connection ()
-        self._use_user = cups.getUser ()
-        self._server = cups.getServer ()
+        cups.setUser (self._use_user)
+        try:
+            self._connection = cups.Connection (host=self._server,
+                                                port=self._port,
+                                                encryption=self._encryption)
+        except TypeError:
+            # Parameters for Connection() require pycups >= 1.9.40.
+            cups.setServer (self._server)
+            cups.setPort (self._port)
+            cups.setEncryption (self._encryption)
+            self._connection = cups.Connection ()
+
         self._user = self._use_user
         debugprint ("Connected as user %s" % self._user)
         methodtype = type (self._connection.getPrinters)
@@ -126,7 +147,13 @@ class Connection:
 
     def _authloop (self, fname, fn, *args, **kwds):
         self._passes = 0
+        c = self._connection
         while self._perform_authentication () != 0:
+            if c != self._connection:
+                # We have reconnected.
+                fn = getattr (self._connection, fname)
+                c = self._connection
+
             try:
                 result = fn.__call__ (*args, **kwds)
 
@@ -142,15 +169,17 @@ class Connection:
                 else:
                     raise
             except cups.HTTPError, (s,):
-                if not self._cancel and s == cups.HTTP_UNAUTHORIZED:
-                    self._failed ()
+                if not self._cancel and (s == cups.HTTP_UNAUTHORIZED or
+                                         s == cups.HTTP_FORBIDDEN):
+                    self._failed (s == cups.HTTP_FORBIDDEN)
                 else:
                     raise
 
         return result
 
-    def _failed (self):
+    def _failed (self, forbidden=False):
         self._has_failed = True
+        self._forbidden = forbidden
 
     def _password_callback (self, prompt):
         debugprint ("Got password callback")
@@ -169,6 +198,7 @@ class Connection:
             # Haven't yet tried the operation.  Set the password
             # callback and return > 0 so we try it for the first time.
             self._has_failed = False
+            self._forbidden = False
             self._auth_called = False
             self._cancel = False
             cups.setPasswordCB (self._password_callback)
@@ -188,16 +218,17 @@ class Connection:
             # Tried the operation without a password and it failed.
             if (self._try_as_root and
                 self._user != 'root' and
-                self._server[0] == '/'):
+                (self._server[0] == '/' or self._forbidden)):
                 # This is a UNIX domain socket connection so we should
-                # not have needed a password, and so the operation must
-                # not be something that the current user is authorised to
-                # do.  They need to try as root, and supply the password.
-                # However, to get the right prompt, we need to try as
-                # root but with no password first.
+                # not have needed a password (or it is not a UDS but
+                # we got an HTTP_FORBIDDEN response), and so the
+                # operation must not be something that the current
+                # user is authorised to do.  They need to try as root,
+                # and supply the password.  However, to get the right
+                # prompt, we need to try as root but with no password
+                # first.
                 debugprint ("Authentication: Try as root")
                 self._use_user = 'root'
-                cups.setUser (self._use_user)
                 self._auth_called = False
                 self._connect ()
                 return 1
@@ -222,14 +253,14 @@ class Connection:
         d.set_auth_info ([self._use_user, ''])
         d.field_grab_focus ('password')
         response = d.run ()
-        d.hide ()
-
-        if response == gtk.RESPONSE_CANCEL:
-            self._cancel = True
-            return -1
-
         (self._use_user,
          self._use_password) = d.get_auth_info ()
+        d.destroy ()
+
+        if (response == gtk.RESPONSE_CANCEL or
+            response == gtk.RESPONSE_DELETE_EVENT):
+            self._cancel = True
+            return -1
 
         cups.setUser (self._use_user)
         debugprint ("Authentication: Reconnect")
