@@ -28,6 +28,9 @@ import os.path
 import re
 from . import _debugprint
 
+__all__ = ['ppdMakeModelSplit',
+           'PPDs']
+
 def ppdMakeModelSplit (ppd_make_and_model):
     """
     Split a ppd-make-and-model string into a canonical make and model pair.
@@ -82,7 +85,7 @@ def ppdMakeModelSplit (ppd_make_and_model):
         model = ppd_make_and_model
     elif re.search ("^(okipage|microline)", \
                       ppd_make_and_model, re.I):
-        make = "Okidata"
+        make = "Oki"
         model = ppd_make_and_model
     elif re.search ("^(konica[\s_-]*minolta)", \
                       ppd_make_and_model, re.I):
@@ -138,6 +141,7 @@ def ppdMakeModelSplit (ppd_make_and_model):
         model = model[:wth]
 
     make = re.sub (r"(?i)KONICA[\s_-]*MINOLTA", "KONICA MINOLTA", make, 1)
+    make = re.sub (r"(?i)HEWLETT[\s_-]PACKARD", "HP", make, 1)
     model = re.sub (r"(?i)\s*\(recommended\)", "", model)
     model = re.sub (r"(?i)\s*-\s*PostScript\b", "", model)
     model = re.sub (r"(?i)\s*\bseries\b", "", model)
@@ -147,9 +151,11 @@ def ppdMakeModelSplit (ppd_make_and_model):
     model = re.sub (r"(?i)\s*\(Bluetooth\)", "", model)
     model = re.sub (r"(?i)\s*-\s*(RC|Ver(|sion))\s*-*\s*[0-9\.]+", "", model)
     model = re.sub (r"(?i)\s*-\s*(RC|Ver(|sion))\b", "", model)
-    model = re.sub (r"(?i)\s*-\s*$", "", model)
+    model = re.sub (r"(?i)\s*PostScript\s*$", "", model)
+    model = re.sub (r"(?i)\s*\(\s*\)", "", model)
+    model = re.sub (r"(?i)\s*[\-\/]\s*$", "", model)
 
-    for mfr in [ "Apple", "Canon", "Epson", "Lexmark", "Okidata" ]:
+    for mfr in [ "Apple", "Canon", "Epson", "Lexmark", "Oki" ]:
         if make == mfr.upper ():
             make = mfr
 
@@ -175,12 +181,13 @@ DRIVER_TYPE_FOOMATIC_GENERIC = 90
 DRIVER_DOES_NOT_WORK = 999
 def _getDriverType (ppdname, ppds=None):
     """Decides which of the above types ppdname is."""
-    if ppdname.startswith ("gutenprint"):
-        if ppdname.find ("/simple/") != -1:
+    if ppdname.find ("gutenprint") != -1:
+        if (ppdname.find ("/simple/") != -1 or
+            ppdname.find (".sim-") != -1):
             return DRIVER_TYPE_GUTENPRINT_NATIVE_SIMPLIFIED
         else:
             return DRIVER_TYPE_GUTENPRINT_NATIVE
-    if ppdname.find ("SpliX")!= -1:
+    if ppdname.find ("splix")!= -1:
         return DRIVER_TYPE_SPLIX
     if (ppdname.find (":") == -1 and
         ppdname.find ("/cups-included/") != -1):
@@ -238,7 +245,6 @@ class PPDs:
 	@param language: language name, as given by the first element
         of the pair returned by locale.getlocale()
         """
-        print _debugprint
         self.ppds = ppds.copy ()
         self.makes = None
         self.ids = None
@@ -311,7 +317,7 @@ class PPDs:
             models_list = self.makes[make].keys ()
         except KeyError:
             return []
-        models_list.sort (cups.modelSort)
+        models_list.sort (key=unicode.lower, cmp=cups.modelSort)
         return models_list
 
     def getInfoFromModel (self, make, model):
@@ -821,10 +827,10 @@ class PPDs:
 def _show_help():
     print "usage: ppds.py [--deviceid] [--list-models] [--list-ids] [--debug]"
 
-def self_test():
+def _self_test(argv):
     import sys, getopt
     try:
-        opts, args = getopt.gnu_getopt (sys.argv[1:], '',
+        opts, args = getopt.gnu_getopt (argv[1:], '',
                                         ['help',
                                          'deviceid',
                                          'list-models',
@@ -888,6 +894,10 @@ def self_test():
     print "\nID matching tests\n"
 
     idlist = [
+        # Format is:
+        # (ID string, max status code, expected driver RE match)
+
+        # Specific models
         ("MFG:EPSON;CMD:ESCPL2,BDC,D4,D4PX;MDL:Stylus D78;CLS:PRINTER;"
          "DES:EPSON Stylus D78;", 1, 'Epson Stylus D68'),
         ("MFG:Hewlett-Packard;MDL:LaserJet 1200 Series;"
@@ -907,6 +917,11 @@ def self_test():
          "DES:Canon iP3000;VER:1.09;STA:10;FSI:03;", 1, "Canon PIXMA iP3000"),
         ("MFG:HP;MDL:Deskjet 5400 series;CMD:MLC,PCL,PML,DW-PCL,DESKJET,DYN;"
          "1284.4DL:4d,4e,1;CLS:PRINTER;DES:5440;", 1, "HP DeskJet 5440"),
+        ("MFG:Hewlett-Packard;MDL:HP LaserJet 3390;"
+         "CMD:PJL,MLC,PCL,POSTSCRIPT,PCLXL;",
+         0, "HP LaserJet 3390.*Postscript"),
+
+        # Generic models
         ("MFG:New;MDL:Unknown PS Printer;CMD:POSTSCRIPT;",
          2, "Generic postscript printer"),
         ("MFG:New;MDL:Unknown PCL6 Printer;CMD:PCLXL;", 2, "Generic PCL 6"),
