@@ -31,6 +31,7 @@ import gtk.glade
 import monitor
 import os
 import pango
+import pwd
 import sys
 import time
 
@@ -139,6 +140,14 @@ class PrinterURIIndex:
 
 
 class JobViewer (monitor.Watcher):
+    required_job_attributes = set(['job-hold-until',
+                                   'job-k-octets',
+                                   'job-name',
+                                   'job-originating-user-name',
+                                   'job-printer-uri',
+                                   'job-state',
+                                   'time-at-creation'])
+
     def __init__(self, bus=None, loop=None, service_running=False,
                  trayicon=False, suppress_icon_hide=False,
                  my_jobs=True, specific_dests=None, exit_handler=None,
@@ -155,16 +164,16 @@ class JobViewer (monitor.Watcher):
         self.jobiters = {}
         self.active_jobs = set() # of job IDs
         self.stopped_job_prompts = set() # of job IDs
-        self.which_jobs = "not-completed"
         self.printer_state_reasons = {}
         self.num_jobs_when_hidden = 0
         self.connecting_to_device = {} # dict of printer->time first seen
         self.state_reason_notifications = {}
+        self.auth_notifications = {} # by job ID
+        self.auth_info_dialogs = {} # by job ID
         self.job_creation_times_timer = None
         self.special_status_icon = False
         self.new_printer_notifications = {}
         self.reasoniters = {}
-        self.auth_info_dialog = None
 
         glade_dir = os.environ.get ("SYSTEM_CONFIG_PRINTER_GLADE",
                                     pkgdata)
@@ -287,8 +296,13 @@ class JobViewer (monitor.Watcher):
         if bus == None:
             bus = dbus.SystemBus ()
 
+        self.host = cups.getServer ()
+        self.port = cups.getPort ()
+        self.encryption = cups.getEncryption ()
         self.monitor = monitor.Monitor (self, bus=bus, my_jobs=my_jobs,
-                                        specific_dests=specific_dests)
+                                        specific_dests=specific_dests,
+                                        host=self.host, port=self.port,
+                                        encryption=self.encryption)
 
         if not self.trayicon:
             self.MainWindow.show ()
@@ -307,6 +321,7 @@ class JobViewer (monitor.Watcher):
     def unset_special_statusicon (self):
         self.special_status_icon = False
         self.statusicon.set_from_pixbuf (self.saved_statusicon_pixbuf)
+        self.set_statusicon_visibility ()
 
     def notify_new_printer (self, printer, notification):
         self.new_printer_notifications[printer] = notification
@@ -319,7 +334,7 @@ class JobViewer (monitor.Watcher):
         notification.attach_to_status_icon (self.statusicon)
         notification.show ()
 
-    def on_new_printer_notification_closed (self, notification):
+    def on_new_printer_notification_closed (self, notification, reason=None):
         printer = notification.get_data ('printer-name')
         del self.new_printer_notifications[printer]
         self.set_statusicon_visibility ()
@@ -366,10 +381,10 @@ class JobViewer (monitor.Watcher):
 
     def on_show_completed_jobs_activate(self, menuitem):
         if menuitem.get_active():
-            self.monitor.which_jobs = "all"
+            which_jobs = "all"
         else:
-            self.monitor.which_jobs = "not-completed"
-        self.monitor.refresh()
+            which_jobs = "not-completed"
+        self.monitor.refresh(which_jobs=which_jobs, refresh_all=False)
 
     def on_show_printer_status_activate(self, menuitem):
         if self.show_printer_status.get_active ():
@@ -454,7 +469,7 @@ class JobViewer (monitor.Watcher):
         store.set_value (iter, 2, data.get('job-name', _("Unknown")))
         self.jobiters[job] = iter
         self.update_job (job, data)
-        self.update_job_creation_times ()
+        store.set_value (iter, 5, _("a minute ago"))
 
     def update_job (self, job, data):
         store = self.store
@@ -470,17 +485,24 @@ class JobViewer (monitor.Watcher):
         store.set_value (iter, 4, size)
 
         state = None
+        job_requires_auth = False
         if data.has_key ('job-state'):
             try:
                 jstate = data['job-state']
                 s = int (jstate)
-                state = { cups.IPP_JOB_PENDING: _("Pending"),
-                          cups.IPP_JOB_HELD: _("Held"),
-                          cups.IPP_JOB_PROCESSING: _("Processing"),
-                          cups.IPP_JOB_STOPPED: _("Stopped"),
-                          cups.IPP_JOB_CANCELED: _("Canceled"),
-                          cups.IPP_JOB_ABORTED: _("Aborted"),
-                          cups.IPP_JOB_COMPLETED: _("Completed") }[s]
+                job_requires_auth = (jstate == cups.IPP_JOB_HELD and
+                                     data.get ('job-hold-until', 'none') ==
+                                     'auth-info-required')
+                if job_requires_auth:
+                    state = _("Held for authentication")
+                else:
+                    state = { cups.IPP_JOB_PENDING: _("Pending"),
+                              cups.IPP_JOB_HELD: _("Held"),
+                              cups.IPP_JOB_PROCESSING: _("Processing"),
+                              cups.IPP_JOB_STOPPED: _("Stopped"),
+                              cups.IPP_JOB_CANCELED: _("Canceled"),
+                              cups.IPP_JOB_ABORTED: _("Aborted"),
+                              cups.IPP_JOB_COMPLETED: _("Completed") }[s]
             except ValueError:
                 pass
             except IndexError:
@@ -491,75 +513,136 @@ class JobViewer (monitor.Watcher):
         store.set_value (iter, 6, state)
 
         # Check whether authentication is required.
-        if (self.trayicon and
-            data['job-state'] == cups.IPP_JOB_HELD and
-            data.get ('job-hold-until', 'none') == 'auth-info-required' and
-            not self.auth_info_dialog):
-            try:
-                cups.require ("1.9.37")
-            except:
-                debugprint ("Authentication required but "
-                            "authenticateJob() not available")
-                return
-
-            # Find out which auth-info is required.
-            try:
-                c = authconn.Connection (self.MainWindow)
+        if self.trayicon:
+            if (job_requires_auth and
+                not self.auth_notifications.has_key (job) and
+                not self.auth_info_dialogs.has_key (job)):
                 try:
-                    uri = data['job-printer-uri']
-                    attributes = c.getPrinterAttributes (uri = uri)
-                except TypeError: # uri keyword introduced in pycups-1.9.32
-                    debugprint ("Fetching printer attributes by name")
-                    attributes = c.getPrinterAttributes (printer)
+                    cups.require ("1.9.37")
+                except:
+                    debugprint ("Authentication required but "
+                                "authenticateJob() not available")
+                    return
+
+                title = _("Authentication Required")
+                text = _("Job requires authentication to proceed.")
+                notification = pynotify.Notification (title, text, 'printer')
+                notification.set_data ('job-id', job)
+                notification.set_urgency (pynotify.URGENCY_NORMAL)
+                notification.set_timeout (pynotify.EXPIRES_NEVER)
+                notification.connect ('closed',
+                                      self.on_auth_notification_closed)
+                self.set_statusicon_visibility ()
+                notification.attach_to_status_icon (self.statusicon)
+                notification.add_action ("authenticate", _("Authenticate"),
+                                         self.on_auth_notification_authenticate)
+                notification.show ()
+                self.auth_notifications[job] = notification
+            elif (not job_requires_auth and
+                  self.auth_notifications.has_key (job)):
+                self.auth_notifications[job].close ()
+
+    def on_auth_notification_closed (self, notification, reason=None):
+        job = notification.get_data ('job-id')
+        debugprint ("auth notification closed for job %s" % job)
+        del self.auth_notifications[job]
+
+    def on_auth_notification_authenticate (self, notification, action):
+        job = notification.get_data ('job-id')
+        debugprint ("auth notification authenticate for job %s" % job)
+        self.display_auth_info_dialog (job)
+
+    def display_auth_info_dialog (self, job):
+        data = self.jobs[job]
+        # Find out which auth-info is required.
+        try:
+            c = authconn.Connection (self.MainWindow,
+                                     host=self.host,
+                                     port=self.port,
+                                     encryption=self.encryption)
+            try:
+                uri = data['job-printer-uri']
+                attributes = c.getPrinterAttributes (uri = uri)
+            except TypeError: # uri keyword introduced in pycups-1.9.32
+                debugprint ("Fetching printer attributes by name")
+                attributes = c.getPrinterAttributes (printer)
+        except cups.IPPError, (e, m):
+            self.show_IPP_Error (e, m)
+            return
+        except RuntimeError:
+            debugprint ("Failed to connect when fetching printer attrs")
+            return
+
+        try:
+            auth_info_required = attributes['auth-info-required']
+        except KeyError:
+            debugprint ("No auth-info-required attribute; guessing instead")
+            auth_info_required = ['username', 'password']
+
+        if not isinstance (auth_info_required, list):
+            auth_info_required = [auth_info_required]
+
+        if auth_info_required == ['negotiate']:
+            # Try Kerberos authentication.
+            try:
+                debugprint ("Trying Kerberos auth for job %d" % jobid)
+                c.authenticateJob (jobid)
+            except TypeError:
+                # Requires pycups-1.9.39 for optional auth parameter.
+                # Treat this as a normal job error.
+                debugprint ("... need newer pycups for that")
+                return
             except cups.IPPError, (e, m):
                 self.show_IPP_Error (e, m)
                 return
-            except RuntimeError:
-                debugprint ("Failed to connect when fetching printer attrs")
-                return
 
+        dialog = authconn.AuthDialog (auth_info_required=auth_info_required)
+        dialog.set_position (gtk.WIN_POS_CENTER)
+
+        # Pre-fill 'username' field.
+        if 'username' in auth_info_required:
             try:
-                auth_info_required = attributes['auth-info-required']
-            except KeyError:
-                debugprint ("No auth-info-required attribute; guessing instead")
-                auth_info_required = ['username', 'password']
+                auth_info = map (lambda x: '', auth_info_required)
+                username = pwd.getpwuid (os.getuid ())[0]
+                ind = auth_info_required.index ('username')
+                auth_info[ind] = username
+                dialog.set_auth_info (auth_info)
 
-            if not isinstance (auth_info_required, list):
-                auth_info_required = [auth_info_required]
+                index = 0
+                for field in auth_info_required:
+                    if auth_info[index] == '':
+                        # Focus on the first empty field.
+                        dialog.field_grab_focus (field)
+                        break
+                    index += 1
+            except:
+                nonfatalException ()
 
-            if auth_info_required == ['negotiate']:
-                # Try Kerberos authentication.
-                try:
-                    debugprint ("Trying Kerberos auth for job %d" % jobid)
-                    c.authenticateJob (jobid)
-                except TypeError:
-                    # Requires pycups-1.9.39 for optional auth parameter.
-                    # Treat this as a normal job error.
-                    debugprint ("... need newer pycups for that")
-                    return
-                except cups.IPPError, (e, m):
-                    self.show_IPP_Error (e, m)
-                    return
+        dialog.set_prompt (_("Authentication required for "
+                             "printing document `%s' (job %d)") %
+                           (data.get('job-name', _("Unknown")), job))
+        self.auth_info_dialogs[job] = dialog
+        dialog.connect ('response', self.auth_info_dialog_response)
+        dialog.connect ('delete-event', self.auth_info_dialog_delete)
+        dialog.set_data ('job-id', job)
+        dialog.show_all ()
 
-            dialog = authconn.AuthDialog (auth_info_required=auth_info_required)
-            dialog.set_prompt (_("Authentication required for "
-                                 "printing document `%s' (job %d)") %
-                               (data.get('job-name', _("Unknown")), job))
-            self.auth_info_dialog = dialog
-            dialog.connect ('response', self.auth_info_dialog_response)
-            dialog.set_data ('job-id', job)
-            dialog.show_all ()
+    def auth_info_dialog_delete (self, dialog, event):
+        self.auth_info_dialog_response (dialog, gtk.RESPONSE_CANCEL)
 
     def auth_info_dialog_response (self, dialog, response):
         dialog.hide ()
-        self.auth_info_dialog = None
+        jobid = dialog.get_data ('job-id')
+        del self.auth_info_dialogs[jobid]
         if response != gtk.RESPONSE_OK:
             return
 
         auth_info = dialog.get_auth_info ()
-        jobid = dialog.get_data ('job-id')
         try:
-            c = authconn.Connection (self.MainWindow)
+            c = authconn.Connection (self.MainWindow,
+                                     host=self.host,
+                                     port=self.port,
+                                     encryption=self.encryption)
             c.authenticateJob (jobid, auth_info)
         except RuntimeError:
             debugprint ("Error connecting to CUPS for authentication")
@@ -635,48 +718,78 @@ class JobViewer (monitor.Watcher):
 
     def on_job_cancel_activate(self, menuitem):
         try:
-            c = authconn.Connection (self.MainWindow)
+            c = authconn.Connection (self.MainWindow,
+                                     host=self.host,
+                                     port=self.port,
+                                     encryption=self.encryption)
             c.cancelJob (self.jobid)
             del c
         except cups.IPPError, (e, m):
-            self.show_IPP_Error (e, m)
+            if (e != cups.IPP_NOT_POSSIBLE and
+                e != cups.IPP_NOT_FOUND):
+                self.show_IPP_Error (e, m)
+            self.monitor.refresh ()
             return
         except RuntimeError:
             return
 
+        self.monitor.refresh ()
+
     def on_job_hold_activate(self, menuitem):
         try:
-            c = authconn.Connection (self.MainWindow)
+            c = authconn.Connection (self.MainWindow,
+                                     host=self.host,
+                                     port=self.port,
+                                     encryption=self.encryption)
             c.setJobHoldUntil (self.jobid, "indefinite")
             del c
         except cups.IPPError, (e, m):
-            self.show_IPP_Error (e, m)
+            if (e != cups.IPP_NOT_POSSIBLE and
+                e != cups.IPP_NOT_FOUND):
+                self.show_IPP_Error (e, m)
+            self.monitor.refresh ()
             return
         except RuntimeError:
             return
 
+        self.monitor.refresh ()
+
     def on_job_release_activate(self, menuitem):
         try:
-            c = authconn.Connection (self.MainWindow)
+            c = authconn.Connection (self.MainWindow,
+                                     host=self.host,
+                                     port=self.port,
+                                     encryption=self.encryption)
             c.setJobHoldUntil (self.jobid, "no-hold")
             del c
         except cups.IPPError, (e, m):
-            self.show_IPP_Error (e, m)
+            if (e != cups.IPP_NOT_POSSIBLE and
+                e != cups.IPP_NOT_FOUND):
+                self.show_IPP_Error (e, m)
+            self.monitor.refresh ()
             return
         except RuntimeError:
             return
 
+        self.monitor.refresh ()
+
     def on_job_reprint_activate(self, menuitem):
         try:
-            c = authconn.Connection (self.MainWindow)
+            c = authconn.Connection (self.MainWindow,
+                                     host=self.host,
+                                     port=self.port,
+                                     encryption=self.encryption)
             c.restartJob (self.jobid)
             del c
         except cups.IPPError, (e, m):
             self.show_IPP_Error (e, m)
+            self.monitor.refresh ()
             return
         except RuntimeError:
             return
 
+        self.monitor.refresh ()
+
     def on_refresh_activate(self, menuitem):
         self.monitor.refresh ()
 
@@ -843,7 +956,7 @@ class JobViewer (monitor.Watcher):
         notification.attach_to_status_icon (self.statusicon)
         notification.show ()
 
-    def on_state_reason_notification_closed (self, notification):
+    def on_state_reason_notification_closed (self, notification, reason=None):
         debugprint ("Notification %s closed" % repr (notification))
         reason = notification.get_data ('printer-state-reason')
         tuple = reason.get_tuple ()
@@ -857,6 +970,7 @@ class JobViewer (monitor.Watcher):
     ## monitor.Watcher interface
     def current_printers_and_jobs (self, mon, printers, jobs):
         self.store.clear ()
+        self.jobs = {}
         self.jobiters = {}
         self.printer_uri_index = PrinterURIIndex (names=printers)
         connection = None
@@ -871,12 +985,37 @@ class JobViewer (monitor.Watcher):
 
             self.add_job (jobid, jobdata)
 
-            # Fetch complete attributes for these jobs.
+            # Fetch required attributes for these jobs.
             attrs = None
+            r = self.required_job_attributes - set (jobdata.keys ())
+            if (jobdata.get ('job-state', cups.IPP_JOB_HELD) !=
+                cups.IPP_JOB_HELD):
+                r -= set (['job-hold-until'])
+
+            if not r:
+                debugprint ("no further attributes required")
+                continue
+
             try:
                 if connection == None:
-                    connection = cups.Connection ()
-                attrs = connection.getJobAttributes (jobid)
+                    try:
+                        connection = cups.Connection (host=self.host,
+                                                      port=self.port,
+                                                      encryption=self.encryption)
+                    except TypeError:
+                        # Requires pycups >= 1.9.40.
+                        cups.setServer (self.host)
+                        cups.setPort (self.port)
+                        cups.setEncryption (self.encryption)
+                        connection = cups.Connection ()
+
+                try:
+                    debugprint ("requesting %s" % r)
+                    attrs = connection.getJobAttributes (jobid,
+                                                         requested_attributes=r)
+                except TypeError:
+                    # requested_attributes requires pycups >= 1.9.42
+                    attrs = connection.getJobAttributes (jobid)
             except RuntimeError:
                 pass
             except AttributeError:
@@ -944,16 +1083,37 @@ class JobViewer (monitor.Watcher):
             # 'auth-info-required'.  This will be checked for in
             # update_job.
             if jobdata['job-state'] == cups.IPP_JOB_HELD:
-                try:
-                    # Fetch the job-hold-until attribute, as this is
-                    # not provided in the notification attributes.
-                    c = cups.Connection ()
-                    attrs = c.getJobAttributes (jobid)
-                    jobdata.update (attrs)
-                except cups.IPPError:
-                    pass
-                except RuntimeError:
-                    pass
+                r = self.required_job_attributes - set (jobdata.keys ())
+                if not r:
+                    debugprint ("no further attributes required")
+                else:
+                    try:
+                        # Fetch the job-hold-until attribute, as this is
+                        # not provided in the notification attributes.
+                        try:
+                            c = cups.Connection (host=self.host,
+                                                 port=self.port,
+                                                 encryption=self.encryption)
+                        except TypeError:
+                            # Requires pycups >= 1.9.40.
+                            cups.setServer (self.host)
+                            cups.setPort (self.port)
+                            cups.setEncryption (self.encryption)
+                            c = cups.Connection ()
+
+                        try:
+                            debugprint ("requesting %s" % r)
+                            attrs = c.getJobAttributes (jobid,
+                                                        requested_attributes=r)
+                        except TypeError:
+                            # requested_attributes requires pycups >= 1.9.42.
+                            attrs = c.getJobAttributes (jobid)
+
+                        jobdata.update (attrs)
+                    except cups.IPPError:
+                        pass
+                    except RuntimeError:
+                        pass
 
             may_be_problem = True
             if (jobdata['job-state'] == cups.IPP_JOB_HELD and
