@@ -40,7 +40,6 @@ if __name__ == "__main__":
 
 import base
 from base import *
-from base import _
 
 class Troubleshooter:
     def __init__ (self, quitfn=None, parent=None):
@@ -109,6 +108,11 @@ class Troubleshooter:
         except:
             self._report_traceback ()
 
+        # Delete the questions so that their __del__ hooks can run.
+        # Do this in reverse order of creation.
+        for i in xrange (len (self.questions)):
+            self.questions.pop ()
+
         self.main.hide ()
         if self.quitfn:
             self.quitfn (self)
@@ -286,6 +290,7 @@ QUESTIONS = ["Welcome",
              "ChoosePrinter",
              "CheckPrinterSanity",
              "CheckPPDSanity",
+             "Locale",
 
              "LocalOrRemote",
              "DeviceListed",
